<?php

class EveryBoard_Article_Helper {

    // Array that contains all "cached" article objects, key is the uuid.
    protected static $board_article_array = array();

    /**
     * Iterates over a board-json and makes ONE request to Open Content to fetch all the manual articles for a board.
     * @param $board_json
     */
    public static function prefetch_board_articles( $board_json ) {

        if ( isset( $board_json ) && array_key_exists( 'rows', $board_json ) ) {

            $oc_api = new OcAPI();
            $article_query = '';
            $number_of_articles = 0;

            foreach ( $board_json['rows'] as $row ) {

                $cols = ( isset( $row['cols'] ) ) ? $row['cols'] : array();
                if ( !empty( $cols ) ) {
                    foreach ( $cols as $col ) {

                        $content = ( isset( $col['content'] ) ) ? $col['content'] : array();
                        foreach ( $content as $content_item ) {

                            if ( isset( $content_item['type'] ) && $content_item['type'] === 'oc_article' ) {
                                $number_of_articles++;
                                $article_query .= 'uuid:' . $content_item['oc_uuid'] . ' OR ';
                            }
                        }
                    }
                }
            }

            // Only prefetch articles if any found in board.
            if ( $article_query !== '' ) {
                $article_query = trim( substr( $article_query, 0, -3 ) );
                $oc_api->prepare_query( $article_query, 0, $number_of_articles );
                $prefetched_articles = $oc_api->get_oc_articles();

                foreach ( $prefetched_articles as $prefetched_article ) {
                    $article_uuid = $prefetched_article->uuid[0];
                    self::$board_article_array[$article_uuid] = $prefetched_article;
                }
            }
        }

    }

    /**
     * @param $uuids
     * @return bool|null
     *
     * Function that prefetches articles from OC by an array of uuids.
     * If it's able to fetch it returns true, otherwise null
     */
    public static function prefetch_board_list_articles( $uuids ) {
        $oc_api = new OcAPI();
        $query = '';

        foreach ( $uuids as $uuid ) {
            $query .= 'uuid:' . $uuid . ' OR ';
        }

        $query = trim( substr( $query, 0, -3 ) );

        $oc_api->prepare_query( "({$query})", 0, count($uuids) );
        $prefetched_articles = $oc_api->get_oc_articles();

        if ( !empty( $prefetched_articles ) ) {
            foreach ( $prefetched_articles as $prefetched_article ) {
                $article_uuid = $prefetched_article->uuid[0];
                self::$board_article_array[$article_uuid] = $prefetched_article;
            }

            return true;
        } else {
            return null;
        }
    }

    /**
     * If the article is in the array it is returned straight from it, otherwise fetch from OC and add to array.
     * @param $uuid
     * @return OcArticle | null
     */
    public static function get_oc_article( $uuid, $force_fetch = true ) {
        $article = null;

        if ( isset( self::$board_article_array[$uuid] ) ) {
            $article = self::$board_article_array[$uuid];
        } else if ( $force_fetch ) {
            $article = self::force_fetch_oc_article( $uuid );

            if ( !is_null( $article ) ) {
                self::$board_article_array[$uuid] = $article;
            }
        }

        return $article;
    }

    /**
     * @param $uuid
     * @return array|null
     *
     * Trying to fetch the article from OC/Cache instead of returning it from the prefetched array
     */
    public static function force_fetch_oc_article( $uuid ) {
        $oc_api = new OcAPI();
        $article = $oc_api->get_single_article( $uuid );

        if ( isset( $article['article'] ) ) {
            $article = $article['article'];
        } else {
            $article = null;
        }

        return $article;
    }

    public static function force_fetch_search_oc_article( $uuid ) {
        $oc_api = new OcAPI();
        $article = $oc_api->search(['q' => "uuid:{$uuid}"], false, true);

        if ( isset( $article['article'] ) ) {
            $article = $article['article'];
        } else {
            $article = null;
        }

        return $article;
    }

}